@@ -7,11 +7,12 @@ pub enum RaknetError{
     NotSupportVersion,
     IncorrectReply,
     PacketParseError,
-    RecvFromError,
+    SocketError,
     IncorrectReliability,
     IncorrectPacketID,
     ReadPacketBufferError,
-    PacketSizeExceedMTU
+    PacketSizeExceedMTU,
+    PacketHeaderError,
 }
 
 pub type Result<T> = std::result::Result<T , RaknetError>;
\ No newline at end of file
