@@ -50,7 +50,6 @@
 //!         if buf[0] == 0xfe{
 //!             //do something
 //!         }
-//!         socket.close().await.unwrap();
 //!     }
 //! }
 //! 
@@ -66,7 +65,6 @@
 //!     if buf[0] == 0xfe{
 //!         //do something
 //!     }
-//!     socket.close().await.unwrap();
 //! }
 //! ```
 
@@ -91,17 +89,20 @@ async fn test_ping_pong(){
     let s = tokio::net::UdpSocket::bind("127.0.0.1:0").await.unwrap();
     let port = s.local_addr().unwrap().port();
 
+    let motd_str = format!("MCPE;Dedicated Server;486;1.18.11;0;10;12322747879247233720;Bedrock level;Survival;1;{};", s.local_addr().unwrap().port());
+
+    let packet = packet::PacketUnconnectedPong{
+        time: utils::cur_timestamp_millis(),
+        magic: true,
+        guid: rand::random(),
+        motd : motd_str.clone()
+    };
+
     tokio::spawn(async move {
         let mut buf = [0u8 ; 1024];
         let (size , addr ) = s.recv_from(&mut buf).await.unwrap();
 
         let _pong = packet::read_packet_ping(&buf[..size]).await.unwrap();
-        let packet = packet::PacketUnconnectedPong{
-            time: utils::cur_timestamp_millis(),
-            magic: true,
-            guid: rand::random(),
-            motd : format!("MCPE;Dedicated Server;486;1.18.11;0;10;12322747879247233720;Bedrock level;Survival;1;{};", s.local_addr().unwrap().port())
-        };
 
         let buf = packet::write_packet_pong(&packet).await.unwrap();
 
@@ -109,57 +110,77 @@ async fn test_ping_pong(){
     });
 
     let addr = format!("127.0.0.1:{}", port);
-    let latency = socket::RaknetSocket::ping(&addr.as_str().parse().unwrap()).await.unwrap();
+    let (latency , motd) = socket::RaknetSocket::ping(&addr.as_str().parse().unwrap()).await.unwrap();
+    assert!(motd_str == motd);
     assert!((0..10).contains(&latency));
 }
 
 #[tokio::test]
 async fn test_connect(){
-    let mut server = RaknetListener::bind("127.0.0.1:0".parse().unwrap()).await.unwrap();
+    let mut server = RaknetListener::bind(&"127.0.0.1:0".parse().unwrap()).await.unwrap();
     let local_addr = server.local_addr().unwrap();
     server.listen().await;
+
+    let notify = std::sync::Arc::new(tokio::sync::Notify::new());
+    let notify2 = notify.clone();
+
     tokio::spawn(async move {
         let mut client1 = server.accept().await.unwrap();
         assert!(client1.local_addr().unwrap() == local_addr);
-        client1.send(&[1,2,3] , Reliability::Reliable).await.unwrap();
+        client1.send(&[0xfe,2,3] , Reliability::Reliable).await.unwrap();
+        notify2.notified().await;
     });
     let mut client2 = RaknetSocket::connect(&local_addr).await.unwrap();
     assert!(client2.peer_addr().unwrap() == local_addr);
     let buf = client2.recv().await.unwrap();
-    assert!(buf == vec![1,2,3]);
+    assert!(buf == vec![0xfe,2,3]);
+
+    notify.notify_one();
 }
 
 #[tokio::test]
 async fn test_send_recv_fragment_data(){
-    let mut server = RaknetListener::bind("127.0.0.1:0".parse().unwrap()).await.unwrap();
+    let mut server = RaknetListener::bind(&"127.0.0.1:0".parse().unwrap()).await.unwrap();
     let local_addr = server.local_addr().unwrap();
     server.listen().await;
+
+    let notify = std::sync::Arc::new(tokio::sync::Notify::new());
+    let notify2 = notify.clone();
+
     tokio::spawn(async move {
         let mut client1 = server.accept().await.unwrap();
         assert!(client1.local_addr().unwrap() == local_addr);
 
         let mut a = vec![3u8;1000];
         let mut b = vec![2u8;1000];
-        let mut c = vec![1u8;1000];
+        let mut c = vec![0xfe;1000];
         b.append(&mut a);
         c.append(&mut b);
 
         client1.send(&c , Reliability::ReliableOrdered).await.unwrap();
+
+        notify2.notified().await;
     });
     let mut client2 = RaknetSocket::connect(&local_addr).await.unwrap();
     assert!(client2.peer_addr().unwrap() == local_addr);
     let buf = client2.recv().await.unwrap();
     assert!(buf.len() == 3000);
-    assert!(buf[0..1000] == vec![1u8;1000]);
+    assert!(buf[0..1000] == vec![0xfe;1000]);
     assert!(buf[1000..2000] == vec![2u8;1000]);
     assert!(buf[2000..3000] == vec![3u8;1000]);
+
+    notify.notify_one();
 }
 
 #[tokio::test]
 async fn test_send_recv_more_reliability_type_packet(){
-    let mut server = RaknetListener::bind("127.0.0.1:0".parse().unwrap()).await.unwrap();
+    let mut server = RaknetListener::bind(&"127.0.0.1:0".parse().unwrap()).await.unwrap();
     let local_addr = server.local_addr().unwrap();
     server.listen().await;
+
+    let notify = std::sync::Arc::new(tokio::sync::Notify::new());
+    let notify2 = notify.clone();
+
     tokio::spawn(async move {
         let mut client1 = server.accept().await.unwrap();
         assert!(client1.local_addr().unwrap() == local_addr);
@@ -178,7 +199,7 @@ async fn test_send_recv_more_reliability_type_packet(){
 
         let mut a = vec![3u8;1000];
         let mut b = vec![2u8;1000];
-        let mut c = vec![1u8;1000];
+        let mut c = vec![0xfe;1000];
         b.append(&mut a);
         c.append(&mut b);
 
@@ -186,13 +207,15 @@ async fn test_send_recv_more_reliability_type_packet(){
 
         let buf = client1.recv().await.unwrap();
         assert!(buf.len() == 3000);
-        assert!(buf[0..1000] == vec![1u8;1000]);
+        assert!(buf[0..1000] == vec![0xfe;1000]);
         assert!(buf[1000..2000] == vec![2u8;1000]);
         assert!(buf[2000..3000] == vec![3u8;1000]);
 
         client1.send(&[0xfe,19,20,21], Reliability::ReliableSequenced).await.unwrap();
         let data = client1.recv().await.unwrap();
         assert!(data == [0xfe,22,23,24].to_vec());
+
+        notify2.notified().await;
     });
     let mut client2 = RaknetSocket::connect(&local_addr).await.unwrap();
     assert!(client2.peer_addr().unwrap() == local_addr);
@@ -214,13 +237,13 @@ async fn test_send_recv_more_reliability_type_packet(){
 
     let buf = client2.recv().await.unwrap();
     assert!(buf.len() == 3000);
-    assert!(buf[0..1000] == vec![1u8;1000]);
+    assert!(buf[0..1000] == vec![0xfe;1000]);
     assert!(buf[1000..2000] == vec![2u8;1000]);
     assert!(buf[2000..3000] == vec![3u8;1000]);
 
     let mut a = vec![3u8;1000];
     let mut b = vec![2u8;1000];
-    let mut c = vec![1u8;1000];
+    let mut c = vec![0xfe;1000];
     b.append(&mut a);
     c.append(&mut b);
 
@@ -230,13 +253,15 @@ async fn test_send_recv_more_reliability_type_packet(){
     assert!(buf == [0xfe,19,20,21]);
 
     client2.send(&[0xfe,22,23,24], Reliability::ReliableSequenced).await.unwrap();
+
+    notify.notify_one();
 }
 
 #[tokio::test]
 async fn test_loss_packet1(){
     let notify = std::sync::Arc::new(tokio::sync::Notify::new());
     let notify2 = notify.clone();
-    let mut server = RaknetListener::bind("127.0.0.1:0".parse().unwrap()).await.unwrap();
+    let mut server = RaknetListener::bind(&"127.0.0.1:0".parse().unwrap()).await.unwrap();
     let local_addr = server.local_addr().unwrap();
     server.listen().await;
     tokio::spawn(async move {
@@ -253,8 +278,8 @@ async fn test_loss_packet1(){
             let data = client1.recv().await.unwrap();
             assert!(data == flag);
         }
-        notify.notify_one();
-
+        
+        notify2.notified().await;
     });
     let mut client2 = RaknetSocket::connect(&local_addr).await.unwrap();
     // 80% loss packet rate
@@ -269,14 +294,14 @@ async fn test_loss_packet1(){
         let data = client2.recv().await.unwrap();
         assert!(data == flag);
     }
-    notify2.notified().await;
+    notify.notify_one();
 }
 
 #[tokio::test]
 async fn test_loss_packet2(){
     let notify = std::sync::Arc::new(tokio::sync::Notify::new());
     let notify2 = notify.clone();
-    let mut server = RaknetListener::bind("127.0.0.1:0".parse().unwrap()).await.unwrap();
+    let mut server = RaknetListener::bind(&"127.0.0.1:0".parse().unwrap()).await.unwrap();
     let local_addr = server.local_addr().unwrap();
     server.listen().await;
     tokio::spawn(async move {
@@ -298,7 +323,7 @@ async fn test_loss_packet2(){
             let data = client1.recv().await.unwrap();
             assert!(data == flag);
         }
-        notify.notify_one();
+        notify2.notified().await;
     });
     let mut client2 = RaknetSocket::connect(&local_addr).await.unwrap();
     // 80% loss packet rate
@@ -318,14 +343,14 @@ async fn test_loss_packet2(){
         let data = client2.recv().await.unwrap();
         assert!(data == flag);
     }
-    notify2.notified().await;
+    notify.notify_one();
 }
 
 #[tokio::test]
 async fn test_loss_packet_with_sequenced(){
     let notify = std::sync::Arc::new(tokio::sync::Notify::new());
     let notify2 = notify.clone();
-    let mut server = RaknetListener::bind("127.0.0.1:0".parse().unwrap()).await.unwrap();
+    let mut server = RaknetListener::bind(&"127.0.0.1:0".parse().unwrap()).await.unwrap();
     let local_addr = server.local_addr().unwrap();
     server.listen().await;
     tokio::spawn(async move {
@@ -349,7 +374,7 @@ async fn test_loss_packet_with_sequenced(){
             assert!(data[1] >= last);
             last = data[1];
         }
-        notify.notify_one();
+        notify2.notified().await;
     });
     let mut client2 = RaknetSocket::connect(&local_addr).await.unwrap();
     // 80% loss packet rate
@@ -372,27 +397,33 @@ async fn test_loss_packet_with_sequenced(){
         last = data[1];
 
     }
-    notify2.notified().await;
+    notify.notify_one();
 }
 
 #[tokio::test]
 async fn test_async_read_write_trait(){
-    let mut server = RaknetListener::bind("127.0.0.1:0".parse().unwrap()).await.unwrap();
+    let mut server = RaknetListener::bind(&"127.0.0.1:0".parse().unwrap()).await.unwrap();
     let local_addr = server.local_addr().unwrap();
     server.listen().await;
+
+    let notify = std::sync::Arc::new(tokio::sync::Notify::new());
+    let notify2 = notify.clone();
+
     tokio::spawn(async move {
         let mut client1 = server.accept().await.unwrap();
         assert!(client1.local_addr().unwrap() == local_addr);
-        tokio::io::AsyncWriteExt::write(&mut client1, &[1,2,3]).await.unwrap();
+        tokio::io::AsyncWriteExt::write(&mut client1, &[0xfe,2,3]).await.unwrap();
         tokio::io::AsyncWriteExt::write(&mut client1, &[0xfe,4,5,6]).await.unwrap();
         tokio::io::AsyncWriteExt::write(&mut client1, &[0xfe,7,8,9]).await.unwrap();
-        tokio::io::AsyncWriteExt::write(&mut client1, &[1,2,3]).await.unwrap();
+        tokio::io::AsyncWriteExt::write(&mut client1, &[0xfe,2,3]).await.unwrap();
+
+        notify2.notified().await;
     });
     let mut client2 = RaknetSocket::connect(&local_addr).await.unwrap();
     assert!(client2.peer_addr().unwrap() == local_addr);
     let mut buf : Vec<u8> = vec![0u8;1];
     tokio::io::AsyncReadExt::read(&mut client2, &mut buf).await.unwrap();
-    assert!(buf == vec![1]);
+    assert!(buf == vec![0xfe]);
     tokio::io::AsyncReadExt::read(&mut client2, &mut buf).await.unwrap();
     assert!(buf == vec![2]);
     tokio::io::AsyncReadExt::read(&mut client2, &mut buf).await.unwrap();
@@ -412,20 +443,24 @@ async fn test_async_read_write_trait(){
 
     let mut buf : Vec<u8> = vec![0u8;3];
     tokio::io::AsyncReadExt::read(&mut client2, &mut buf).await.unwrap();
-    assert!(buf == vec![1,2,3]);
+    assert!(buf == vec![0xfe,2,3]);
+
+    notify.notify_one();
 }
 
 /*
 #[tokio::test]
 async fn chore(){
-    let mut client = RaknetSocket::connect(&"192.168.199.127:19132".parse().unwrap()).await.unwrap();
+    let mut server = RaknetListener::bind(&"127.0.0.1:19132".parse().unwrap()).await.unwrap();
+    server.listen().await;
+    let mut client = RaknetSocket::connect(&"127.0.0.1:19132".parse().unwrap()).await.unwrap();
     let mut a = vec![3u8;1000];
     let mut b = vec![2u8;1000];
     let mut c = vec![0xfe;1000];
     b.append(&mut a);
     c.append(&mut b);
     client.send(&c, Reliability::ReliableOrdered).await.unwrap();
-    client.recv().await.unwrap();
+    std::mem::drop(client);
 }
 
 #[tokio::test]
