@@ -149,7 +149,6 @@ impl RaknetSocket {
     /// if buf[0] == 0xfe{
     ///    //do something
     /// }
-    /// socket.close().await.unwarp(); // you need to manually close raknet connection
     /// ```
     pub async fn connect(addr : &SocketAddr) -> Result<Self>{
 
@@ -541,15 +540,6 @@ impl RaknetSocket {
     }
 
     /// Close Raknet Socket
-    /// 
-    /// The Raknet Socket needs to be closed manually, and if no valid data packets are received for more than 1 minute, the Raknet connection will be closed automatically. 
-    /// This method can be called repeatedly.
-    /// 
-    /// # Example
-    /// ```ignore
-    /// let mut socket = RaknetSocket::connect("127.0.0.1:19132".parse().unwrap()).await.unwrap();
-    /// socket.close().await.unwarp();
-    /// ```
     pub async fn close(&mut self) -> Result<()>{
 
         if self.connected.load(Ordering::Relaxed){
@@ -559,51 +549,62 @@ impl RaknetSocket {
         Ok(())
     }
 
-    /// Unconnected ping a Raknet Server and return latency
+    /// Unconnected ping a Raknet Server and return latency and motd.
     /// 
     /// # Example
     /// ```ignore
-    /// let latency = socket::RaknetSocket::ping("127.0.0.1:19132".parse().unwrap()).await.unwrap();
+    /// let (latency, motd) = socket::RaknetSocket::ping("127.0.0.1:19132".parse().unwrap()).await.unwrap();
     /// assert!((0..10).contains(&latency));
     /// ```
-    pub async fn ping(addr : &SocketAddr) -> Result<i64> {
-        let packet = PacketUnconnectedPing{
-            time: cur_timestamp_millis(),
-            magic: true,
-            guid: rand::random(),
-        };
+    pub async fn ping(addr : &SocketAddr) -> Result<(i64 , String)> {
+
 
         let s = match UdpSocket::bind("0.0.0.0:0").await{
             Ok(p) => p,
             Err(_) => return Err(RaknetError::BindAdreesError),
         };
 
-        let buf = write_packet_ping(&packet).await?;
-
         loop{
+
+            let packet = PacketUnconnectedPing{
+                time: cur_timestamp_millis(),
+                magic: true,
+                guid: rand::random(),
+            };
+
+            let buf = write_packet_ping(&packet).await?;
+
             match s.send_to(buf.as_slice(), addr).await{
-                Ok(_) => break,
+                Ok(_) => {},
                 Err(e) => {
                     raknet_log_error!("udp socket sendto error {}" , e);
-                    tokio::time::sleep(std::time::Duration::from_secs(2)).await;
-                    continue;
+                    return Err(RaknetError::SocketError);
                 },
             };
+
+            let mut buf = [0u8 ; 1024];
+
+            match match tokio::time::timeout(std::time::Duration::from_secs(5), s.recv_from(&mut buf)).await{
+                Ok(p) => p,
+                Err(_) => {
+                    continue;
+                },
+            }{
+                Ok(p) => p,
+                Err(_) => return Err(RaknetError::SocketError),
+            };
+    
+            match read_packet_pong(&buf).await{
+                Ok(p) => {
+                    return Ok((p.time - packet.time , p.motd))
+                },
+                Err(_) => {},
+            };
+
+            tokio::time::sleep(std::time::Duration::from_secs(2)).await;
         }
 
-
-        let mut buf = [0u8 ; 1024];
-        match s.recv_from(&mut buf).await{
-            Ok(p) => p,
-            Err(_) => return Err(RaknetError::RecvFromError),
-        };
-
-        let pong = match read_packet_pong(&buf).await{
-            Ok(p) => p,
-            Err(_) => return Err(RaknetError::PacketParseError),
-        };
-
-        Ok(pong.time - packet.time)
+        
     }
 
     /// Send a packet
@@ -619,6 +620,14 @@ impl RaknetSocket {
     /// ```
     pub async fn send(&mut self , buf : &[u8] , r : Reliability) ->Result<()> {
 
+        if buf.len() < 1 {
+            return Err(RaknetError::PacketHeaderError);
+        }
+
+        if buf[0] != 0xfe{
+            return Err(RaknetError::PacketHeaderError);
+        }
+
         if !self.connected.load(Ordering::Relaxed){
             return Err(RaknetError::ConnectionClosed);
         }
@@ -655,7 +664,7 @@ impl RaknetSocket {
                 if !self.connected.load(Ordering::Relaxed){
                     return Err(RaknetError::ConnectionClosed);
                 }
-                Err(RaknetError::RecvFromError)
+                Err(RaknetError::SocketError)
             },
         }
 
@@ -766,6 +775,14 @@ impl AsyncWrite for RaknetSocket {
 
         match self.sendq.lock().boxed_local().poll_unpin(cx){
             std::task::Poll::Ready(mut p) => {
+                if buf.len() < 1 {
+                    return std::task::Poll::Ready(Err(std::io::Error::new(std::io::ErrorKind::InvalidData, "packet header must be 0xfe")));
+                }
+        
+                if buf[0] != 0xfe{
+                    return std::task::Poll::Ready(Err(std::io::Error::new(std::io::ErrorKind::InvalidData, "packet header must be 0xfe")));
+                }
+
                 p.insert(Reliability::ReliableOrdered, buf).unwrap();
                 std::task::Poll::Ready(Ok(buf.len()))
             },
@@ -781,4 +798,10 @@ impl AsyncWrite for RaknetSocket {
     fn poll_shutdown(self: std::pin::Pin<&mut Self>, _cx: &mut std::task::Context<'_>) -> std::task::Poll<std::io::Result<()>> {
         std::task::Poll::Ready(Ok(()))
     }
+}
+
+impl Drop for RaknetSocket{
+    fn drop(&mut self) {
+        futures::executor::block_on(self.close()).unwrap();
+    }
 }
\ No newline at end of file
